﻿using ProjectMonorail.Entities;
using ProjectMonorail.MovementStrategy;

namespace ProjectMonorail.DrawingObjects
{
    /// <summary>
    /// Класс, отвечающий за прорисовку и перемещение объекта-сущности
    /// </summary>
    public class DrawingMonorail
    {
        /// <summary>
        /// Класс-сущность
        /// </summary>
        public EntityMonorail? EntityMonorail { get; protected set; }

        /// <summary>
        /// Ширина окна
        /// </summary>
        private int _pictureWidth;

        public int PictureWidth
        {
            set { _pictureWidth = value; }
        }

        /// <summary>
        /// Высота окна
        /// </summary>
        private int _pictureHeight;

        public int PictureHeight
        {
            set { _pictureHeight = value; }
        }

        /// <summary>
        /// Левая координата прорисовки монорельса
        /// </summary>
        protected int _startPosX;

        /// <summary>
        /// Верхняя координата прорисовки монорельса
        /// </summary>
        protected int _startPosY;

        /// <summary>
        /// Ширина прорисовки монорельса
        /// </summary>
        protected int _monorailWidth = 117;

        /// <summary>
        /// Высота прорисовки монорельса
        /// </summary>
        protected int _monorailHeight = 56;

        /// <summary>
        /// Координата X объекта
        /// </summary>
        public int GetPosX => _startPosX;

        /// <summary>
        /// Координата Y объекта
        /// </summary>
        public int GetPosY => _startPosY;

        /// <summary>
        /// Ширина объекта
        /// </summary>
        public int GetWidth => _monorailWidth;

        /// <summary>
        /// Высота объекта
        /// </summary>
        public int GetHeight => _monorailHeight;

        /// <summary>
        /// Получение объекта IMoveableObject из объекта DrawingMonorail
        /// </summary>
        public IMoveableObject GetMoveableObject => new DrawingObjectMonorail(this);

        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес</param>
        /// <param name="mainColor">Основной цвет</param>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        public DrawingMonorail(int speed, double weight, Color mainColor, int width, int height)
        {
            if (width < _monorailWidth || height < _monorailHeight) { return; }
            _pictureWidth = width;
            _pictureHeight = height;
            EntityMonorail = new EntityMonorail(speed, weight, mainColor);
        }

        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес</param>
        /// <param name="mainColor">Основной цвет</param>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        /// <param name="monorailWidth">Ширина прорисовки монорельса</param>
        /// <param name="monorailHeight">Высота прорисовки монорельса</param>
        protected DrawingMonorail(int speed, double weight, Color mainColor, int width,
            int height, int monorailWidth, int monorailHeight)
        {
            if (width < monorailWidth || height < monorailHeight) { return; }
            _pictureWidth = width;
            _pictureHeight = height;
            _monorailWidth = monorailWidth;
            _monorailHeight = monorailHeight;
            EntityMonorail = new EntityMonorail(speed, weight, mainColor);
        }

        /// <summary>
        /// Установка позиции
        /// </summary>
        /// <param name="x">Координата X</param>
        /// <param name="y">Координата Y</param>
        public void SetPosition(int x, int y)
        {
            if (x < 0 || x + _monorailWidth > _pictureWidth) { x = 0; }
            if (y < 0 || y + _monorailHeight > _pictureHeight) { y = 0; }

            _startPosX = x;
            _startPosY = y;
        }

        /// <summary>
        /// Проверка, что объект может переместится по указанному направлению
        /// </summary>
        /// <param name="direction">Направление</param>
        /// <returns>true - можно переместится по указанному направлению</returns>
        public bool CanMove(DirectionType direction)
        {
            if (EntityMonorail == null)
            {
                return false;
            }

            return direction switch
            {
                //влево
                DirectionType.Left => _startPosX - EntityMonorail.Step > 0,
                //вверх
                DirectionType.Up => _startPosY - EntityMonorail.Step > 0,
                //вправо
                DirectionType.Right => _startPosX + _monorailWidth + EntityMonorail.Step < _pictureWidth,
                //вниз
                DirectionType.Down => _startPosY + _monorailHeight + EntityMonorail.Step < _pictureHeight,
                _ => false
            };
        }

        /// <summary>
        /// Изменение направления перемещения
        /// </summary>
        /// <param name="direction">Направление</param>
        public void MoveTransport(DirectionType direction)
        {
            if (!CanMove(direction) || EntityMonorail == null)
            {
                return;
            }
            switch (direction)
            {
                //влево
                case DirectionType.Left:
                    _startPosX -= (int)EntityMonorail.Step;
                    break;
                //вверх
                case DirectionType.Up:
                    _startPosY -= (int)EntityMonorail.Step;
                    break;
                //вправо
                case DirectionType.Right:
                    _startPosX += (int)EntityMonorail.Step;
                    break;
                //вниз
                case DirectionType.Down:
                    _startPosY += (int)EntityMonorail.Step;
                    break;
            }
        }

        /// <summary>
        /// Прорисовка объекта
        /// </summary>
        /// <param name="g"></param>
        public virtual void DrawTransport(Graphics g)
        {
            if (EntityMonorail == null)
            {
                return;
            }
            Pen mainPen = new Pen(Color.Black, 2);
            Pen additionalPen = new(Color.Blue);
            Brush mainBrush = new SolidBrush(EntityMonorail.MainColor);
            Brush brBlue = new SolidBrush(Color.Blue);
            Brush brBlack = new SolidBrush(Color.Black);
            Brush brWhite = new SolidBrush(Color.White);
            Brush brGray = new SolidBrush(Color.Gray);

            //надстройка
            g.FillRectangle(mainBrush, _startPosX + 55, _startPosY, 25, 15);
            g.DrawRectangle(mainPen, _startPosX + 55, _startPosY, 25, 15);

            //корпус локомотива
            Point[] locoPoints = { new Point(_startPosX + 29, _startPosY + 15), new Point(_startPosX + 112, _startPosY + 15),
            new Point(_startPosX + 112, _startPosY + 46), new Point(_startPosX + 25, _startPosY + 46), new Point(_startPosX + 25, _startPosY + 31) };
            g.FillPolygon(mainBrush, locoPoints);
            g.DrawPolygon(mainPen, locoPoints);
            g.DrawLine(additionalPen, _startPosX + 25, _startPosY + 31, _startPosX + 112, _startPosY + 31);

            //дверь локомотива
            g.FillRectangle(brGray, _startPosX + 54, _startPosY + 21, 7, 20);
            g.DrawRectangle(mainPen, _startPosX + 54, _startPosY + 21, 7, 20);

            //окна локомотива
            g.FillRectangle(brBlue, _startPosX + 32, _startPosY + 18, 6, 9);
            g.DrawRectangle(mainPen, _startPosX + 32, _startPosY + 18, 6, 9);
            g.FillRectangle(brBlue, _startPosX + 44, _startPosY + 18, 6, 9);
            g.DrawRectangle(mainPen, _startPosX + 44, _startPosY + 18, 6, 9);
            g.FillRectangle(brBlue, _startPosX + 103, _startPosY + 18, 6, 9);
            g.DrawRectangle(mainPen, _startPosX + 103, _startPosY + 18, 6, 9);

            //колеса и тележка локомотива 
            g.FillRectangle(brBlack, _startPosX + 23, _startPosY + 47, 33, 6);
            g.DrawRectangle(mainPen, _startPosX + 23, _startPosY + 47, 33, 6);
            g.FillRectangle(brBlack, _startPosX + 76, _startPosY + 47, 30, 6);
            g.DrawRectangle(mainPen, _startPosX + 76, _startPosY + 47, 30, 6);
            g.FillEllipse(brWhite, _startPosX + 25, _startPosY + 47, 10, 9);
            g.DrawEllipse(mainPen, _startPosX + 25, _startPosY + 47, 10, 9);
            g.FillEllipse(brWhite, _startPosX + 45, _startPosY + 47, 10, 9);
            g.DrawEllipse(mainPen, _startPosX + 45, _startPosY + 47, 10, 9);
            g.FillEllipse(brWhite, _startPosX + 75, _startPosY + 47, 10, 9);
            g.DrawEllipse(mainPen, _startPosX + 75, _startPosY + 47, 10, 9);
            g.FillEllipse(brWhite, _startPosX + 95, _startPosY + 47, 10, 9);
            g.DrawEllipse(mainPen, _startPosX + 95, _startPosY + 47, 10, 9);
            Point[] bogiePoints = { new Point(_startPosX + 26, _startPosY + 46), new Point(_startPosX + 24, _startPosY + 54),
            new Point(_startPosX + 12, _startPosY + 54), new Point(_startPosX + 8, _startPosY + 51),  new Point(_startPosX + 12, _startPosY + 48),
            new Point(_startPosX + 18, _startPosY + 46) };
            g.FillPolygon(brBlack, bogiePoints);

            //соединение между кабинами
            g.DrawRectangle(mainPen, _startPosX + 112, _startPosY + 18, 5, 28);
            g.FillRectangle(brBlack, _startPosX + 112, _startPosY + 18, 5, 28);
        }
    }
}
