﻿using ProjectMonorail.DrawingObjects;
using ProjectMonorail.Generics;
using System.Windows.Forms;

namespace ProjectMonorail
{
    /// <summary>
    /// Форма для работы с набором объектов класса DrawingMonorail
    /// </summary>
    public partial class FormMonorailCollection : Form
    {
        /// <summary>
        /// Набор объектов
        /// </summary>
        private readonly MonorailsGenericStorage _storage;

        /// <summary>
        /// Конструктор
        /// </summary>
        public FormMonorailCollection()
        {
            InitializeComponent();
            _storage = new MonorailsGenericStorage(pictureBoxCollection.Width, pictureBoxCollection.Height);
        }

        /// <summary>
        /// Заполнение listBoxSets
        /// </summary>
        private void ReloadObjects()
        {
            int index = listBoxSets.SelectedIndex;
            listBoxSets.Items.Clear();
            for (int i = 0; i < _storage.Keys.Count; i++)
            {
                listBoxSets.Items.Add(_storage.Keys[i]);
            }
            if (listBoxSets.Items.Count > 0 && (index == -1 || index >= listBoxSets.Items.Count))
            {
                listBoxSets.SelectedIndex = 0;
            }
            else if (listBoxSets.Items.Count > 0 && index > -1 && index < listBoxSets.Items.Count)
            {
                listBoxSets.SelectedIndex = index;
            }
        }

        /// <summary>
        /// Добавление набора в коллекцию
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonAddSet_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxNameSet.Text))
            {
                MessageBox.Show("Не все данные заполнены", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            _storage.AddSet(textBoxNameSet.Text);
            ReloadObjects();
        }

        /// <summary>
        /// Выбор набора
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void listBoxSets_SelectedIndexChanged(object sender, EventArgs e)
        {
            pictureBoxCollection.Image =
            _storage[listBoxSets.SelectedItem?.ToString() ?? string.Empty]?.ShowMonorails();
        }

        /// <summary>
        /// Удаление набора
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonDelSet_Click(object sender, EventArgs e)
        {
            if (listBoxSets.SelectedIndex == -1)
            {
                return;
            }
            if (MessageBox.Show($"Удалить набор {listBoxSets.SelectedItem}?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                _storage.DelSet(listBoxSets.SelectedItem.ToString() ?? string.Empty);
                ReloadObjects();
            }
        }

        private void addMonorail(DrawingMonorail monorail)
        {
            monorail.PictureHeight = pictureBoxCollection.Height;
            monorail.PictureWidth = pictureBoxCollection.Width;
            if (listBoxSets.SelectedIndex == -1)
            {
                return;
            }

            var obj = _storage[listBoxSets.SelectedItem.ToString() ?? string.Empty];
            if (obj == null)
            {
                return;
            }

            if (obj + monorail != -1)
            {
                MessageBox.Show("Объект добавлен");
                pictureBoxCollection.Image = obj.ShowMonorails();
            }
            else
            {
                MessageBox.Show("Не удалось добавить объект");
            }
        }

        /// <summary>
        /// Добавление объекта в набор
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonAddMonorail_Click(object sender, EventArgs e)
        {
            if (listBoxSets.SelectedIndex == -1)
            {
                return;
            }
            var formMonorailConfig = new FormMonorailConfig();
            formMonorailConfig.AddEvent(addMonorail);
            formMonorailConfig.Show();
        }

        /// <summary>
        /// Удаление объекта из набора
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonRemoveMonorail_Click(object sender, EventArgs e)
        {
            if (listBoxSets.SelectedIndex == -1)
            {
                return;
            }

            var obj = _storage[listBoxSets.SelectedItem.ToString() ?? string.Empty];
            if (obj == null)
            {
                return;
            }

            if (MessageBox.Show("Удалить объект?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }

            int pos = Convert.ToInt32(maskedTextBoxNumber.Text);

            if (obj - pos)
            {
                MessageBox.Show("Объект удален");
                pictureBoxCollection.Image = obj.ShowMonorails();
            }
            else
            {
                MessageBox.Show("Не удалось удалить объект");
            }
        }

        /// <summary>
        /// Обновление рисунка по набору
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonRefreshCollection_Click(object sender, EventArgs e)
        {
            if (listBoxSets.SelectedIndex == -1)
            {
                return;
            }

            var obj = _storage[listBoxSets.SelectedItem.ToString() ?? string.Empty];
            if (obj == null)
            {
                return;
            }

            pictureBoxCollection.Image = obj.ShowMonorails();
        }

        /// <summary>
        /// Обработка нажатия "Сохранение"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SaveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                if (_storage.SaveData(saveFileDialog.FileName))
                {
                    MessageBox.Show("Save was successful",
                    "Result", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
                else
                {
                    MessageBox.Show("Save failed", "Result",
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        /// <summary>
        /// Обработка нажатия "Загрузка"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LoadToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (openFileDialog.ShowDialog() == DialogResult.OK)
            {
                if (_storage.LoadData(openFileDialog.FileName))
                {
                    MessageBox.Show("Load was successful",
                    "Result", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    ReloadObjects();
                }
                else
                {
                    MessageBox.Show("Load failed", "Result",
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }
    }
}
