﻿using ProjectMonorail.DrawingObjects;
using ProjectMonorail.MovementStrategy;
using System.Text;

namespace ProjectMonorail.Generics
{
    /// <summary>
    /// Класс для хранения коллекции
    /// </summary>
    internal class MonorailsGenericStorage
    {
        /// <summary>
        /// Разделитель для записи ключа и значения элемента словаря
        /// </summary>
        private static readonly char _separatorForKeyValue = '|';

        /// <summary>
        /// Разделитель для записей коллекции данных в файл
        /// </summary>
        private readonly char _separatorRecords = ';';

        /// <summary>
        /// Разделитель для записи информации по объекту в файл
        /// </summary>
        private static readonly char _separatorForObject = ':';

        /// <summary>
        /// Словарь (хранилище)
        /// </summary>
        readonly Dictionary<string, MonorailsGenericCollection<DrawingMonorail, DrawingObjectMonorail>> _monorailsStorages;

        /// <summary>
        /// Возвращение списка названий наборов
        /// </summary>
        public List<string> Keys => _monorailsStorages.Keys.ToList();

        /// <summary>
        /// Ширина окна отрисовки
        /// </summary>
        private readonly int _pictureWidth;

        /// <summary>
        /// Высота окна отрисовки
        /// </summary>
        private readonly int _pictureHeight;

        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="pictureWidth"></param>
        /// <param name="pictureHeight"></param>
        public MonorailsGenericStorage(int pictureWidth, int pictureHeight)
        {
            _monorailsStorages = new Dictionary<string, MonorailsGenericCollection<DrawingMonorail, DrawingObjectMonorail>>();
            _pictureWidth = pictureWidth;
            _pictureHeight = pictureHeight;
        }

        /// <summary>
        /// Добавление набора
        /// </summary>
        /// <param name="name">Название набора</param>
        public void AddSet(string name)
        {
            if (!_monorailsStorages.ContainsKey(name))
                _monorailsStorages.Add(name, new MonorailsGenericCollection<DrawingMonorail, DrawingObjectMonorail>(_pictureWidth, _pictureHeight));
        }

        /// <summary>
        /// Удаление набора
        /// </summary>
        /// <param name="name">Название набора</param>
        public void DelSet(string name)
        {
            if (_monorailsStorages.ContainsKey(name))
                _monorailsStorages.Remove(name);
        }

        /// <summary>
        /// Доступ к набору
        /// </summary>
        /// <param name="ind"></param>
        /// <returns></returns>
        public MonorailsGenericCollection<DrawingMonorail, DrawingObjectMonorail>? this[string ind]
        {
            get
            {
                if (_monorailsStorages.ContainsKey(ind))
                    return _monorailsStorages[ind];
                return null;
            }
        }

        /// <summary>
        /// Сохранение информации по монорельсам в хранилище в файл
        /// </summary>
        /// <param name="filename">Путь и имя файла</param>
        /// <returns>true - сохранение прошло успешно, false - ошибка при сохранении данных</returns>
        public bool SaveData(string filename)
        {
            if (File.Exists(filename))
            {
                File.Delete(filename);
            }
            StringBuilder data = new();
            foreach (KeyValuePair<string, MonorailsGenericCollection<DrawingMonorail, DrawingObjectMonorail>> record in _monorailsStorages)
            {
                StringBuilder records = new();
                foreach (DrawingMonorail? elem in record.Value.GetMonorails)
                {
                    records.Append($"{elem?.GetDataForSave(_separatorForObject)}{_separatorRecords}");
                }
                data.AppendLine($"{record.Key}{_separatorForKeyValue}{records}");
            }
            if (data.Length == 0)
            {
                return false;
            }
            using StreamWriter sw = new(filename);
            sw.Write($"MonorailStorage{Environment.NewLine}{data}");
            return true;
        }

        /// <summary>
        /// Загрузка информации по монорельсам в хранилище из файла
        /// </summary>
        /// <param name="filename">Путь и имя файла</param>
        /// <returns>true - загрузка прошла успешно, false - ошибка при загрузке данных</returns>
        public bool LoadData(string filename)
        {
            if (!File.Exists(filename))
            {
                return false;
            }
            using (StreamReader sr = new(filename))
            {
                string str = sr.ReadLine();
                if (str == null || str.Length == 0)
                {
                    return false;
                }
                if (!str.StartsWith("MonorailStorage"))
                {
                    //если нет такой записи, то это не те данные
                    return false;
                }
                _monorailsStorages.Clear();
                while ((str = sr.ReadLine()) != null)
                {
                    string[] record = str.Split(_separatorForKeyValue, StringSplitOptions.RemoveEmptyEntries);
                    if (record.Length != 2)
                    {
                        continue;
                    }
                    MonorailsGenericCollection<DrawingMonorail, DrawingObjectMonorail> collection = new(_pictureWidth, _pictureHeight);
                    string[] set = record[1].Split(_separatorRecords, StringSplitOptions.RemoveEmptyEntries);
                    foreach (string elem in set.Reverse())
                    {
                        DrawingMonorail? monorail = elem?.CreateDrawingMonorail(_separatorForObject, _pictureWidth, _pictureHeight);
                        if (monorail != null)
                        {
                            if (collection + monorail == -1)
                            {
                                return false;
                            }
                        }
                    }
                    _monorailsStorages.Add(record[0], collection);
                }
            }
            return true;
        }
    }
}
