﻿namespace ProjectMonorail.Generics
{
    /// <summary>
    /// Параметризованный набор объектов
    /// </summary>
    /// <typeparam name="T"></typeparam>
    internal class SetGeneric<T> where T : class
    {
        /// <summary>
        /// Список объектов, которые храним
        /// </summary>
        private readonly List<T?> _places;

        /// <summary>
        /// Количество объектов в списке
        /// </summary>
        public int Count => _places.Count;

        /// <summary>
        /// Максимальное количество объектов в списке
        /// </summary>
        private readonly int _maxCount;

        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="count"></param>
        public SetGeneric(int count)
        {
            _maxCount = count;
            _places = new List<T?>(count);
        }

        /// <summary>
        /// Добавление объекта в набор
        /// </summary>
        /// <param name="monorail">Добавляемый монорельс</param>
        /// <returns></returns>
        public int Insert(T monorail)
        {
            return Insert(monorail, 0);
        }

        /// <summary>
        /// Добавление объекта в набор на конкретную позицию
        /// </summary>
        /// <param name="monorail">Добавляемый монорельс</param>
        /// <param name="position">Позиция</param>
        /// <returns></returns>
        public int Insert(T monorail, int position)
        {   
            if (position < 0 || position > Count || Count >= _maxCount)
                return -1;

            _places.Insert(position, monorail);
            return position;
        }

        /// <summary>
        /// Удаление объекта из набора с конкретной позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public bool Remove(int position)
        {
            if (position < 0 || position >= Count)
                return false;
            _places.RemoveAt(position);
            return true;
        }

        /// <summary>
        /// Получение объекта из набора по позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T? this[int position]
        {
            get
            {
                if (position < 0 || position >= Count)
                    return null;
                return _places[position];
            }
            set
            {
                if (position < 0 || position > Count || Count >= _maxCount)
                    return;

                _places.Insert(position, value);
            }
        }

        /// <summary>
        /// Проход по списку
        /// </summary>
        /// <returns></returns>
        public IEnumerable<T?> GetMonorails(int? maxMonorails = null)
        {
            for (int i = 0; i < _places.Count; ++i)
            {
                yield return _places[i];
                if (maxMonorails.HasValue && i == maxMonorails.Value)
                {
                    yield break;
                }
            }
        }
    }
}
